<?php

namespace Drupal\config_pages_overrides\Config;

use Drupal\Component\Utility\NestedArray;
use Drupal\config_inspector\ConfigInspectorManager;
use Drupal\config_pages\ConfigPagesLoaderServiceInterface;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\ConfigFactoryOverrideInterface;
use Drupal\Core\Config\StorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Configuration overrides for the profile.
 *
 * @package Drupal\stanford_profile_config_overrides\Config
 */
class ConfigOverrides implements ConfigFactoryOverrideInterface {

  /**
   * Config Pages service.
   *
   * @var \Drupal\config_pages\ConfigPagesLoaderServiceInterface
   */
  protected $configPagesLoader;

  /**
   * Config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Existing config pages types.
   *
   * @var \Drupal\config_pages\ConfigPagesTypeInterface[]
   */
  protected $configPagesTypes;

  /**
   * Config Inspector manager service.
   *
   * @var \Drupal\config_inspector\ConfigInspectorManager
   */
  protected $configInspectorManager;

  /**
   * ConfigOverrides constructor.
   *
   * @param \Drupal\config_pages\ConfigPagesLoaderServiceInterface $config_pages
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Drupal\config_inspector\ConfigInspectorManager $config_manager
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(ConfigPagesLoaderServiceInterface $config_pages, EntityTypeManagerInterface $entity_type_manager, ConfigInspectorManager $config_manager) {
    $this->configPagesLoader = $config_pages;
    $this->configPagesTypes = $entity_type_manager->getStorage('config_pages_type')
      ->loadMultiple();
    $this->configInspectorManager = $config_manager;
  }

  /**
   * {@inheritDoc}
   */
  public function loadOverrides($names) {
    $overrides = [];
    foreach ($this->configPagesTypes as $page_type) {
      foreach ($page_type->getThirdPartySettings('config_pages_overrides') as $settings) {
        if (in_array($settings['config_name'], $names)) {
          $path = [$settings['config_name']];
          $path = array_merge($path, explode('.', $settings['config_item']));
          $override_value = $this->configPagesLoader->getValue($page_type->id(), $settings['field'], $settings['delta'], $settings['column']);

          if (!is_null($override_value)) {
            $override_value = $this->castOverrideValue($override_value, $settings['config_name'], $settings['config_item']);

            NestedArray::setValue($overrides, $path, $override_value);
          }
        }
      }
    }
    return $overrides;
  }

  /**
   * Convert the override value based on the schema definition.
   *
   * @param mixed $value
   *   Override value.
   * @param string $config_name
   *   Configuration name.
   * @param $config_item
   *   Path of configuration item.
   *
   * @return mixed
   *   Casted value.
   */
  protected function castOverrideValue($value, $config_name, $config_item) {
    $config_schema = $this->configInspectorManager->getConfigSchema($config_name);
    $schema = $this->configInspectorManager->convertConfigElementToList($config_schema);
    if (!isset($schema[$config_item])) {
      return $value;
    }

    switch ($schema[$config_item]->getDataDefinition()['type']) {
      case 'boolean':
        return (bool) $value;

      case 'integer':
        return (int) $value ?: strlen((string) $value);
    }

    return $value;
  }

  /**
   * {@inheritDoc}
   */
  public function createConfigObject($name, $collection = StorageInterface::DEFAULT_COLLECTION) {
    return NULL;
  }

  /**
   * {@inheritDoc}
   */
  public function getCacheableMetadata($name) {
    return new CacheableMetadata();
  }

  /**
   * {@inheritDoc}
   */
  public function getCacheSuffix() {
    return 'ConfigPagesOverrides';
  }

}
